@@ -1,4 +1,3 @@
 from decouple import config
 
-root = config('WXBHOME')
-
+root = config("WXBHOME")
