@@ -6,9 +6,10 @@ import numpy as np
 import torch as th
 import torch.nn as nn
 
-from wxbtool.data.constants import load_area_weight, load_lsm, load_slt, load_orography, load_lat2d, load_lon2d
-from wxbtool.util.evaluation import Evaluator
+from wxbtool.data.constants import (load_area_weight, load_lat2d, load_lon2d,
+                                    load_lsm, load_orography, load_slt)
 from wxbtool.data.dataset import WxDataset, WxDatasetClient
+from wxbtool.util.evaluation import Evaluator
 
 
 def cast(element):
@@ -22,39 +23,6 @@ class Model2d(nn.Module):
         super().__init__()
         self.setting = setting
 
-        lsm = cast(load_lsm(setting.resolution, setting.root))
-        slt = cast(load_slt(setting.resolution, setting.root))
-        oro = cast(load_orography(setting.resolution, setting.root))
-        aw = cast(load_area_weight(setting.resolution, setting.root))
-
-        phi = cast(load_lat2d(setting.resolution, setting.root)) * np.pi / 180
-        theta = cast(load_lon2d(setting.resolution, setting.root)) * np.pi / 180
-        x, y = np.meshgrid(np.linspace(0, 1, num=32), np.linspace(0, 1, num=64))
-        x = cast(x)
-        y = cast(y)
-
-        lsm.requires_grad = False
-        slt.requires_grad = False
-        oro.requires_grad = False
-        aw.requires_grad = False
-        phi.requires_grad = False
-        theta.requires_grad = False
-        x.requires_grad = False
-        y.requires_grad = False
-
-        dt = th.cos(phi)
-        self.weight = dt / dt.mean()
-
-        lsm = ((lsm - 0.33707827) / 0.45900375).view(1, 1, 32, 64)
-        slt = ((slt - 0.67920434) / 1.1688842).view(1, 1, 32, 64)
-        oro = ((oro - 379.4976) / 859.87225).view(1, 1, 32, 64)
-        self.constant = th.cat((lsm, slt, oro), dim=1)
-        self.constant_size = self.constant.size()[1]
-        self.phi = phi
-        self.theta = theta
-        self.x = x
-        self.y = y
-
         self.constant_cache = {}
         self.weight_cache = {}
         self.phi_cache = {}
@@ -76,73 +44,165 @@ class Model2d(nn.Module):
 
         self.clipping_threshold = 3.0
 
+        # Assume lsm, slt, oro as constant inputs
+        self._constant_size = 3
+
+    def prepare_constant(self):
+        lsm = cast(load_lsm(self.setting.resolution, self.setting.root))
+        slt = cast(load_slt(self.setting.resolution, self.setting.root))
+        oro = cast(load_orography(self.setting.resolution, self.setting.root))
+        aw = cast(load_area_weight(self.setting.resolution, self.setting.root))
+
+        phi = cast(load_lat2d(self.setting.resolution, self.setting.root)) * np.pi / 180
+        theta = (
+            cast(load_lon2d(self.setting.resolution, self.setting.root)) * np.pi / 180
+        )
+        x, y = np.meshgrid(np.linspace(0, 1, num=32), np.linspace(0, 1, num=64))
+        x = cast(x)
+        y = cast(y)
+
+        lsm.requires_grad = False
+        slt.requires_grad = False
+        oro.requires_grad = False
+        aw.requires_grad = False
+        phi.requires_grad = False
+        theta.requires_grad = False
+        x.requires_grad = False
+        y.requires_grad = False
+
+        dt = th.cos(phi)
+        self.weight = dt / dt.mean()
+
+        lsm = ((lsm - 0.33707827) / 0.45900375).view(1, 1, 32, 64)
+        slt = ((slt - 0.67920434) / 1.1688842).view(1, 1, 32, 64)
+        oro = ((oro - 379.4976) / 859.87225).view(1, 1, 32, 64)
+        self.constant = th.cat((lsm, slt, oro), dim=1)
+        self.phi = phi
+        self.theta = theta
+        self.x = x
+        self.y = y
+
     def load_dataset(self, phase, mode, **kwargs):
-        if phase == 'train':
-            if mode == 'server':
-                self.dataset_train, self.dataset_eval = (
-                        WxDataset(self.setting.root, self.setting.resolution,
-                                  self.setting.years_train, self.setting.vars, self.setting.levels,
-                                  input_span=self.setting.input_span, pred_shift=self.setting.pred_shift, pred_span=self.setting.pred_span, step=self.setting.step),
-                        WxDataset(self.setting.root, self.setting.resolution,
-                                  self.setting.years_eval, self.setting.vars, self.setting.levels,
-                                  input_span=self.setting.input_span, pred_shift=self.setting.pred_shift, pred_span=self.setting.pred_span, step=self.setting.step)
-                )
-            else:
-                ds_url = kwargs['url']
-                self.dataset_train, self.dataset_eval = (
-                    WxDatasetClient(ds_url, 'train', self.setting.resolution,
-                              self.setting.years_train, self.setting.vars, self.setting.levels,
-                              input_span=self.setting.input_span, pred_shift=self.setting.pred_shift,
-                              pred_span=self.setting.pred_span, step=self.setting.step),
-                    WxDatasetClient(ds_url, 'eval', self.setting.resolution,
-                              self.setting.years_eval, self.setting.vars, self.setting.levels,
-                              input_span=self.setting.input_span, pred_shift=self.setting.pred_shift,
-                              pred_span=self.setting.pred_span, step=self.setting.step)
-                )
-
-            self.train_size = len(self.dataset_train)
-            self.eval_size = len(self.dataset_eval)
+        if mode == "server":
+            self.dataset_train, self.dataset_eval, self.dataset_test = (
+                WxDataset(
+                    self.setting.root,
+                    self.setting.resolution,
+                    self.setting.years_train,
+                    self.setting.vars,
+                    self.setting.levels,
+                    input_span=self.setting.input_span,
+                    pred_shift=self.setting.pred_shift,
+                    pred_span=self.setting.pred_span,
+                    step=self.setting.step,
+                ),
+                WxDataset(
+                    self.setting.root,
+                    self.setting.resolution,
+                    self.setting.years_eval,
+                    self.setting.vars,
+                    self.setting.levels,
+                    input_span=self.setting.input_span,
+                    pred_shift=self.setting.pred_shift,
+                    pred_span=self.setting.pred_span,
+                    step=self.setting.step,
+                ),
+                WxDataset(
+                    self.setting.root,
+                    self.setting.resolution,
+                    self.setting.years_test,
+                    self.setting.vars,
+                    self.setting.levels,
+                    input_span=self.setting.input_span,
+                    pred_shift=self.setting.pred_shift,
+                    pred_span=self.setting.pred_span,
+                    step=self.setting.step,
+                ),
+            )
         else:
-            if mode == 'server':
-                self.dataset_test = WxDataset(self.setting.root, self.setting.resolution,
-                                              self.setting.years_test, self.setting.vars, self.setting.levels,
-                                              input_span=self.setting.input_span, pred_shift=self.setting.pred_shift, pred_span=self.setting.pred_span, step=self.setting.step)
-            else:
-                ds_url = kwargs['url']
-                self.dataset_test = WxDatasetClient(ds_url, 'test', self.setting.resolution,
-                                              self.setting.years_test, self.setting.vars, self.setting.levels,
-                                              input_span=self.setting.input_span, pred_shift=self.setting.pred_shift,
-                                              pred_span=self.setting.pred_span, step=self.setting.step)
+            ds_url = kwargs["url"]
+            self.dataset_train, self.dataset_eval, self.dataset_test = (
+                WxDatasetClient(
+                    ds_url,
+                    "train",
+                    self.setting.resolution,
+                    self.setting.years_train,
+                    self.setting.vars,
+                    self.setting.levels,
+                    input_span=self.setting.input_span,
+                    pred_shift=self.setting.pred_shift,
+                    pred_span=self.setting.pred_span,
+                    step=self.setting.step,
+                ),
+                WxDatasetClient(
+                    ds_url,
+                    "eval",
+                    self.setting.resolution,
+                    self.setting.years_eval,
+                    self.setting.vars,
+                    self.setting.levels,
+                    input_span=self.setting.input_span,
+                    pred_shift=self.setting.pred_shift,
+                    pred_span=self.setting.pred_span,
+                    step=self.setting.step,
+                ),
+                WxDatasetClient(
+                    ds_url,
+                    "test",
+                    self.setting.resolution,
+                    self.setting.years_test,
+                    self.setting.vars,
+                    self.setting.levels,
+                    input_span=self.setting.input_span,
+                    pred_shift=self.setting.pred_shift,
+                    pred_span=self.setting.pred_span,
+                    step=self.setting.step,
+                ),
+            )
 
-            self.test_size = len(self.dataset_test)
+        self.train_size = len(self.dataset_train)
+        self.eval_size = len(self.dataset_eval)
+        self.test_size = len(self.dataset_test)
 
     def get_constant(self, input, device):
         if device not in self.constant_cache:
+            if not hasattr(self, "constant"):
+                self.prepare_constant()
             self.constant_cache[device] = self.constant.to(device)
         return self.constant_cache[device]
 
     def get_weight(self, device):
         if device not in self.weight_cache:
+            if not hasattr(self, "weight"):
+                self.prepare_constant()
             self.weight_cache[device] = self.weight.to(device)
         return self.weight_cache[device]
 
     def get_phi(self, device):
         if device not in self.phi_cache:
+            if not hasattr(self, "phi"):
+                self.prepare_constant()
             self.phi_cache[device] = self.phi.to(device)
         return self.phi_cache[device]
 
     def get_theta(self, device):
         if device not in self.theta_cache:
+            if not hasattr(self, "theta"):
+                self.prepare_constant()
             self.theta_cache[device] = self.theta.to(device)
         return self.theta_cache[device]
 
     def get_x(self, device):
         if device not in self.x_cache:
+            if not hasattr(self, "x"):
+                self.prepare_constant()
             self.x_cache[device] = self.x.to(device)
         return self.x_cache[device]
 
     def get_y(self, device):
         if device not in self.y_cache:
+            if not hasattr(self, "y"):
+                self.prepare_constant()
             self.y_cache[device] = self.y.to(device)
         return self.y_cache[device]
 
@@ -156,6 +216,11 @@ class Model2d(nn.Module):
             self.grid_polr_cache[device] = self.grid_polr.to(device)
         return self.grid_polr_cache[device]
 
+    def constant_size(self):
+        if not hasattr(self, "_constant_size"):
+            self.prepare_constant()
+        return self._constant_size
+
 
 class Base2d(Model2d):
     def __init__(self, setting, enable_da=False):
@@ -175,7 +240,7 @@ class Base2d(Model2d):
             augmented = []
             b, c, w, h = data.size()
             for _ in range(b):
-                slice = data[_:_+1]
+                slice = data[_ : _ + 1]
                 shift = self.lng_shift[_]
                 flip = self.flip_status[_]
                 slice = slice.roll(shift, dims=(3,))
@@ -186,7 +251,9 @@ class Base2d(Model2d):
         return data
 
     def get_augmented_constant(self, input):
-        constant = self.get_constant(input, input.device).repeat(input.size()[0], 1, 1, 1)
+        constant = self.get_constant(input, input.device).repeat(
+            input.size()[0], 1, 1, 1
+        )
         constant = self.augment_data(constant)
         phi = self.get_phi(input.device).repeat(input.size()[0], 1, 1, 1)
         theta = self.get_theta(input.device).repeat(input.size()[0], 1, 1, 1)
@@ -223,7 +290,7 @@ class Base3d(Base2d):
             augmented = []
             b, c, t, w, h = data.size()
             for _ in range(b):
-                slice = data[_:_+1]
+                slice = data[_ : _ + 1]
                 shift = self.lng_shift[_]
                 flip = self.flip_status[_]
                 slice = slice.roll(shift, dims=(4,))
