@@ -1,18 +1,17 @@
 import numpy as np
 import torch as th
 
-
-from wxbtool.phys.constants import WaterDensity, IceDensity
+from wxbtool.phys.constants import IceDensity, WaterDensity
 
 
 def water_mass_by_density(d):
-    l = d / 1000
-    return np.pi * l * l * l / 6 * WaterDensity
+    diameter = d / 1000
+    return np.pi * diameter * diameter * diameter / 6 * WaterDensity
 
 
 def ice_mass_by_density(d):
-    l = d / 1000
-    return np.pi * l * l * l / 6 * IceDensity
+    diameter = d / 1000
+    return np.pi * diameter * diameter * diameter / 6 * IceDensity
 
 
 def water_reflectivity_by_density(d):
