@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import numpy as np
+
 import wxbtool.data.constants as cnst
 
 
@@ -9,7 +10,7 @@ class Evaluator:
         self.root = root
         self.resolution = resolution
 
-        if resolution == '5.625deg':
+        if resolution == "5.625deg":
             self.levels = 11
             self.width = 32
             self.length = 64
